"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var Excel = __importStar(require("exceljs"));
var workbook;
var flat_1 = require("flat");
var maxDepth = 0;
var sheet;
var headerInformation = {};
var cellTracker = {};
var delimiter = '.';
function getSampleJson(json) {
    if (typeof json === 'object' && Array.isArray(json)) {
        if (json.length <= 0) {
            return {};
        }
        return json[0];
    }
    return json;
}
function addWorkSheet(workSheet) {
    var worksheetTittle = workSheet.title;
    var worksheetOptions = (workSheet === null || workSheet === void 0 ? void 0 : workSheet.options) || {};
    return workbook.addWorksheet(worksheetTittle, worksheetOptions);
}
function setExcelHeader(flattenJson, headerFormatter) {
    headerInformation = getHeaderInformation(flattenJson);
    for (var header in headerInformation) {
        if (headerInformation.hasOwnProperty(header)) {
            mergeCell(header, headerInformation[header], headerFormatter);
        }
    }
}
function getHeaderInformation(flattenJson) {
    for (var data in flattenJson) {
        if (flattenJson.hasOwnProperty(data)) {
            var splittedArray = data.split(delimiter);
            var lastHeaderKey = splittedArray[splittedArray.length - 1];
            var rowSpan = 0;
            var previousHeaderKey = "";
            for (var i = 0; i < splittedArray.length; i++) {
                var headerKey = splittedArray[i];
                var rowNumber = i + 1;
                var headerKeyCopy = "" + headerKey + delimiter + previousHeaderKey;
                if (!(headerInformation === null || headerInformation === void 0 ? void 0 : headerInformation[headerKeyCopy]) ||
                    ((headerInformation === null || headerInformation === void 0 ? void 0 : headerInformation[headerKeyCopy]) && headerInformation[headerKeyCopy].rowNumber !== rowNumber)) {
                    if (lastHeaderKey === headerKey && i === splittedArray.length - 1) {
                        rowSpan = maxDepth - rowNumber - 1;
                    }
                    headerInformation[headerKeyCopy] = {
                        colSpan: 0,
                        rowSpan: rowSpan,
                        rowNumber: rowNumber,
                    };
                }
                else {
                    headerInformation[headerKeyCopy].colSpan += 1;
                }
                previousHeaderKey = headerKeyCopy;
            }
        }
    }
    return headerInformation;
}
function mergeCell(header, _a, headerFormatter) {
    var colSpan = _a.colSpan, rowNumber = _a.rowNumber, rowSpan = _a.rowSpan;
    var startRow = rowNumber;
    var startColumn = getColumnCell(rowNumber, colSpan, rowSpan);
    var endRow = startRow + rowSpan;
    var endColumn = startColumn + colSpan;
    sheet.mergeCells(startRow, startColumn, endRow, endColumn);
    var row = sheet.getRow(rowNumber);
    var cell = row.getCell(startColumn);
    cell.value = headerFormatter ? headerFormatter(header.split(delimiter)[0]) : header.split(delimiter)[0];
}
function getColumnCell(rowNumber, colSpan, rowSpan) {
    var startColumn = (cellTracker === null || cellTracker === void 0 ? void 0 : cellTracker[rowNumber]) ? cellTracker[rowNumber] + 1 : 1;
    for (var i = rowNumber; i <= rowNumber + rowSpan; i++) {
        cellTracker[i] = startColumn + colSpan;
    }
    return startColumn;
}
function findMaxDepth(flattenJson) {
    Object.keys(flattenJson).forEach(function (data) {
        var splittedArray = data.split(delimiter);
        if (maxDepth <= splittedArray.length) {
            maxDepth = splittedArray.length + 1;
        }
    });
}
module.exports = {
    generateExcel: function (sheetConfigurations) {
        workbook = new Excel.Workbook();
        if (!Array.isArray(sheetConfigurations)) {
            sheetConfigurations = [sheetConfigurations];
        }
        var checkSheetConfiguration = function (sheetConfiguration) {
            if (!(sheetConfiguration === null || sheetConfiguration === void 0 ? void 0 : sheetConfiguration.title)) {
                throw new Error('Sheet title is missing in one of the sheet object');
            }
            if (!(sheetConfiguration === null || sheetConfiguration === void 0 ? void 0 : sheetConfiguration.data)) {
                throw new Error('Sheet data is missing in one of the sheet object');
            }
        };
        sheetConfigurations.forEach(function (sheetConfig) {
            var _a, _b, _c;
            delimiter = (_b = (_a = sheetConfigurations[0]) === null || _a === void 0 ? void 0 : _a.delimiter) !== null && _b !== void 0 ? _b : '.';
            cellTracker = {};
            maxDepth = 0;
            headerInformation = {};
            var headerFormatterFn = (_c = sheetConfigurations[0]) === null || _c === void 0 ? void 0 : _c.headerFormatter;
            sheet = addWorkSheet(sheetConfig);
            checkSheetConfiguration(sheetConfig);
            var data = Array.isArray(sheetConfig === null || sheetConfig === void 0 ? void 0 : sheetConfig.data) ? sheetConfig === null || sheetConfig === void 0 ? void 0 : sheetConfig.data : [sheetConfig === null || sheetConfig === void 0 ? void 0 : sheetConfig.data];
            var flattenJson = (0, flat_1.flatten)(getSampleJson(data), {
                delimiter: delimiter,
            });
            findMaxDepth(flattenJson);
            setExcelHeader(flattenJson, headerFormatterFn);
            sheet.columns = Object.keys(flattenJson).map(function (jsonKey) { return ({
                key: jsonKey,
            }); });
            data.forEach(function (jsonData) {
                sheet.addRow((0, flat_1.flatten)(jsonData, {
                    delimiter: delimiter,
                }));
            });
        });
        return workbook;
    },
};
